#!/usr/bin/env python
# -*- coding: utf-8 -*- 

import wx
import sys
import matplotlib
matplotlib.use('WXAgg')
import DectFrame
import detector


class App(wx.App):
    def OnInit(self):
        '''Create the main window and insert the custom frame'''
        frame = DectFrame.create(None, file_name)
        self.SetTopWindow(frame)
        frame.Show(True)
        return True

if __name__=='__main__':
    if len(sys.argv)>1:
    	file_name = sys.argv[1]
    else:
    	file_name = None
    app = App(0)
    app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*- 
import os
import sys
import time
import ffmpeg

import numpy as np
import pandas as pd
import trackpy as tp
import subprocess as sp
from scipy.stats import linregress
from scipy.signal import find_peaks,peak_prominences

import matplotlib.pyplot as plt
# from matplotlib.cm import Greys_r, coolwarm,viridis
import matplotlib.cm as cm
from matplotlib.colors import LinearSegmentedColormap

# import warnings
# warnings.simplefilter(action='ignore', category=FutureWarning)


### To do
## [ ] ArgParse: Option to do all or undone
## [ ] Input/output for lists of files processed/with errors
## [X] Close plot window after saving



class detector(object):
    '''Particle detection platform for identifying the group climbing velocity of a 
    group of flies (or particles) in a Drosophila negative geotaxis (climbing) assay.
    This platform is designed to take videos of varying background homogeneity and 
    extract the x,y,time coordinates of flies by subtracting the static portion of the 
    video. Climbing velocities are calculated as the most linear portion of a user-defined
    subset of frames--done so by vial if defined. 
    '''
    def __init__(self, video_file, config_file = None,gui=False, variables = None):
        '''Initializing detector object'''
        print('---------------> Initializing detector')
        self.video_file = self.check_video(video_file)
        
        if gui:
            self.load_for_gui(variables = variables)
        elif  not gui:
            self.load_for_main(config_file = config_file)

#         for item in variables:
#             print(item)

        print('\n\n')
        self.specify_paths_details(video_file)
        self.image_stack = self.video_to_array(video_file,loglevel='panic')
        return


## Loading functions    
    def load_for_gui(self,variables):
        print('---------------> GUI: loading variables for a GUI run')
        self.config = None
        self.path_project = None

        if variables == None:
            print('\n\nExiting program. No variable list loaded')
            raise SystemExit
        for item in variables:
#             print(item)
            try: exec('self.'+item)
            except: pass
        return     
        
    def load_for_main(self, config_file = None):
        print('---------------> MAIN: loading variables for command line run')
        if config_file == None:
            self.config_file = sys.argv[1]
        self.check_cfg(self.config_file)
        self.load_from_cfg(self.config_file)
        return
        
    def load_from_cfg(self,config_file=None):
        print('---------------> load_from_cfg: Reading in variables from .cfg file')
        try:
            with open(config_file,'r') as f:
                variables = f.read().split('\n')
            f.close()
            
            variables = [item for item in variables if not item.startswith(('#',' ','\n'))]
            for item in variables:
                try: exec('self.'+item)
                except: pass
        except: 
            print('\n\nExiting program. Could not read in file.cfg, but path and suffix are good')
            raise SystemExit
        return

## Checking functions
    def check_video(self,video_file=None):
        print('---------------> Confirming video path is a file')
        if os.path.isfile(video_file): pass
        else:
            print('\n\nExiting program. Invalid path to video file: ',video_file)
            raise SystemExit
        return video_file
        
    def check_cfg(self,config_file=None):
        print('---------------> Check_cfg: confirming .cfg file is a valid path')
        if os.path.isfile(config_file) and config_file.endswith('.cfg'): pass
        else:
            print('\n\nExiting program. Not a valid configuration file')
            raise SystemExit
        return

## Specifying variables
    def specify_paths_details(self,video_file):
        print('---------------> specify_paths_details: creating variables from those already loaded')
        folder,name = os.path.split(video_file)
        self.name = name[:-5]
        self.name_nosuffix = '.'.join(video_file.split('.')[:-1])
        self.experiment_details = self.name.split('_')
        self.file_details = dict(zip(self.naming_convention.split('_'),self.name.split('_')))
        
        ## Defining final file names and destinations
        folder_names = ['data','filter','plot','slope']
        folder_suffixes = ['.raw.csv','.filter.csv','.diag.png','.slopes.csv']
        for item,jtem in zip(folder_names,folder_suffixes):
            var_name = 'self.path_'+item
            file_path = os.path.join(folder,name+jtem)
            exec(var_name+"='"+file_path+"'")
        
        ## Project folder specific paths
        if self.path_project == None: self.path_project = os.path.join(folder,self.name + '.cfg')
#         self.path_result  = os.path.join(self.path_project,'results.csv')

        ## Naming individual vials
        self.experiment_details[-1] = '.'.join(self.experiment_details[-1].split('.')[:-1])
        self.vial_ID = self.experiment_details[:self.vvars]
        self.discrete_cmap(vials=self.vials)
#         print(self.color_list)
        return

## Video processing functions
    def video_to_array(self, file, **kwargs):
        print('---------------> video_to_array: Convert video to nd-array')
        probe = ffmpeg.probe(file)
        video_info = next(x for x in probe['streams'] if x['codec_type'] == 'video')
        self.width = int(video_info['width'])
        self.height = int(video_info['height'])

        out,err = (ffmpeg
                   .input(file)
                   .output('pipe:',format='rawvideo', pix_fmt='rgb24',**kwargs)
                   .run(capture_stdout=True))

        self.n_frames = int(len(out)/self.height/self.width/3)
        image_stack = np.frombuffer(out, np.uint8).reshape([-1, self.height, self.width, 3])
        return image_stack


    def crop_n_grayscale(self,video_array,
                         y=0,y_max=None,
                         x=0,x_max=None,
                         first_frame=None,
                         last_frame=None):
        print('---------------> Processing video:')
    
        ## Conditional for cropping video length
        if first_frame == None: first_frame = 0
        if last_frame == None: last_frame = video_array.shape[0]
        if y_max == None: y_max = video_array.shape[2]
        if x_max == None: x_max = video_array.shape[1]
        print('        - Cropping from frame %s to %s' % (first_frame,last_frame))
    
        ## Setting only frames and ROI to grayscale
        print('        - Converting to grayscale & cropping ROI to (%s x %s)' % (x_max-x,y_max-y))
        ch_1 = 0.2989 * video_array[first_frame:last_frame,y : y_max,x : x_max,0]
        ch_2 = 0.5870 * video_array[first_frame:last_frame,y : y_max,x : x_max,1]
        ch_3 = 0.1140 * video_array[first_frame:last_frame,y : y_max,x : x_max,2]
        clean_stack = ch_1.astype(float) + ch_2.astype(float) + ch_3.astype(float)

        print('  Final video array dimensions:',clean_stack.shape,'\n')
        return clean_stack
        
    def subtract_background(self,video_array=None,first_frame=0,last_frame=None):
        print('----> Calculate a background image')
        if last_frame == None: 
            last_frame = self.n_frames
#         if video_array == None: 
#             print('Exiting program. Needs an appropriate image stack to subtract background')
            
        background = np.median(video_array[first_frame:last_frame,:,:].astype(float), axis=0).astype(int)
        print('  subtract_background dimensions:     ', background.shape)
        print('')
        spot_stack = np.subtract(video_array,background)
        return spot_stack, background   


## Plots and views
    def view_ROI(self,image=None,border=True,x0=0,x1=None,y0=0,y1=None,**kwargs):
        print('---------------> view_ROI: View image with Region of Interest identified')

#         if image == None:
#             image = self.image_stack[0]

        plt.imshow(self.image_stack[0],cmap=cm.Greys_r, **kwargs)
        
        if border:
            if x1 == None: x1 = self.image_stack[0].shape[1]
            if y1 == None: y1 = self.image_stack[0].shape[0]
            plt.hlines(y0,x0,x1, color = 'r', alpha = .7)
            plt.hlines(y1,x0,x1, color = 'r', alpha = .7)
            plt.vlines(x0,y0,y1, color = 'r', alpha = .7)
            plt.vlines(x1,y0,y1, color = 'r', alpha = .7)
        return


    def display_images(self, cropped_converted, background,subtracted, frame=0, **kwargs):
        print('----> Displaying images')
        plt.figure(figsize = (6,8))
    
        ## Displaying the test frame image
        plt.subplot(311)
        plt.title('Cropped and converted, frame: %s' % str(frame))
        plt.imshow(cropped_converted[frame], cmap = cm.Greys_r, **kwargs)

        ## Displaying the background image
        plt.subplot(312)
        plt.title('Background image')
        plt.imshow(background, cmap = cm.Greys_r, **kwargs)

        ## Displaying the background subtracted, test frame image
        plt.subplot(313)
        plt.title('Subtracted background')
        plt.imshow(subtracted[frame], cmap = cm.Greys_r, **kwargs)

        plt.tight_layout()
        print('')
        return


    def image_metrics(self, spots, image, metric, colorbar=False, **kwargs):
        plt.title(metric)
        plt.imshow(image, cmap = cm.Greys_r)
        plt.scatter(spots.x,spots.y, c = spots[metric],
                    cmap = cm.coolwarm, **kwargs)
        if colorbar:
            plt.colorbar()
        plt.tight_layout()
        return


    def colored_hist(self, spots,metric,bins=None, predict_threshold=False, threshold=None):
        if bins == None: bins = 40
        try: threshold = int(threshold)
        except: pass
    
        set_cm = plt.cm.get_cmap('coolwarm')
        n, bin_assignments, patches = plt.hist(spots[metric],bins = bins)
        bin_centers = 0.5 * (bin_assignments[:-1] + bin_assignments[1:])
        col = bin_centers - min(bin_centers)
        col /= max(col)

        ## Plotting by color
        for c, p in zip(col, patches):
            plt.setp(p, 'facecolor', set_cm(c))
    
        ## Estimate auto-threshold
        if predict_threshold:
            _threshold = self.find_threshold(spots[metric],bins = bins)
            y_max = np.histogram(spots[metric],bins=bins)[0].max()
            plt.vlines(_threshold,0,y_max,color = 'gray',label='Auto')
        if isinstance(threshold, int) or isinstance(threshold, float):
            y_max = np.histogram(spots[metric],bins=bins)[0].max()
            plt.vlines(threshold,0,y_max,color = 'k', label='User-defined')
            if predict_threshold:
                plt.legend(frameon=False)
        return

    def spot_checker(self, spots,metrics=['signal'],image=None,**kwargs):
        ## Plotting each of the detector spot results over the image

        subplot = int(str(len(metrics)) + str(2) + str(0))
        if image==None: image = self.clean_stack[0]
        count = 0
        plt.figure(figsize=(4+image.shape[1]/150,len(metrics)*2))
        for i in range(0,len(metrics)):
            ## Defining spot metric and whether to auto-threshold
            col = metrics[i]
            if col=='signal': predict = True
            else: predict = False

            ## Drawing histogram, showing distribution of spots by metric
            count += 1
            plt.subplot(len(metrics),2,count)
            plt.title('Histogram for: %s' % col)
            self.colored_hist(spots,metric=col, bins = 40,predict_threshold=predict)
    
            ## Drawing image plot, colored by metric
            count += 1
            plt.subplot(len(metrics),2,count)
            plt.title('Spot overlay: %s' % col)
            self.image_metrics(spots,image, metric=col,**kwargs)

        plt.tight_layout()
        return
        
    def visualize_metrics(self,spots):
        metrics = ['ecc','mass','signal']
        self.spot_checker(spots,metrics=metrics, alpha=.1)
        return


## Spot finding
    def find_spots(self, stack, diameter = 3, quiet=True,**kwargs):
        ## Check diameter
        diameter = int(diameter)
        if ~diameter % 2: diameter = diameter + 1
    
        ## Option to silence output
        if quiet: tp.quiet()
    
        ## Detect spots
        spots = tp.batch(stack,diameter = diameter, **kwargs)

        ## Sorting DataFrame
        return spots[spots.raw_mass > 0].sort_values(by='frame')
        
        
    def particle_finder(self, invert=True, **kwargs): ## Break this into two functions, since parameter_testing function uses similar components
        print('---------------> particle_finder: Finds particles and formats resulting dataframe')
        df = self.find_spots(stack = self.spot_stack,
                                      quiet=True,invert=True,
                                      diameter=self.diameter,
                                      minmass=self.minmass,
                                      maxsize=self.maxsize)
        if df.shape[0] == 0:
            print('\n\nExiting program: No spots detected. Try modifying diameter, maxsize, or minmass parameters')
            raise SystemExit
        
        ## Round detector outputs
        df['x'] = df.x.round(2)
        df['y'] = df.y.round(2)
        df['t'] = [round(item/self.frame_rate,3) for item in df.frame]
        df['mass'] = df['mass'].astype(int)
        df['size'] = df.size.round(4)
        df['ecc'] = df.ecc.round(4)
        df['signal'] = df.signal.round(2)
        df['raw_mass'] = df['mass'].astype(int)
        df['ep'] = df.ep.round(1)
        df['True_particle'] = np.repeat(True,df.shape[0])#(df['signal'] > threshold)
        
        ## Saves DataFrame
        return df

    def find_threshold(self,x_array,bins=40):
        print('----> Selecting a threshold')
        x_array = np.histogram(x_array,bins = bins)[0]
        peaks = find_peaks(x_array)[0]
        prominences = peak_prominences(x_array,peaks)
        threshold = find_peaks(x_array, prominence=np.max(prominences))[0]
        print('    Threshold @',str(threshold[0]))
        print('')
        return threshold[0]

    def invert_Y(self,spots):
        return abs(spots.y - spots.y.max())
    
    def create_df_by_vial(self):
        '''
        Creates a dictionary with keys for vials and values for the DataFrame sliced by
        vial. It will also calculate the local linear regression for each vial and
        returns the DataFrame containing all the slopes and linear regression statistics
        for that vial.
        ----
        Inputs:
        df (DataFrame) : DataFrame sliced by vial
        vials (int) : Number of vials in video
        window (int) : Window size to calculate local linear regression
        vial_ID (str) : Vial-specific ID, taken from the first 'vvars' of naming convention
        ----
        Returns:
        result (dict) : DataFrame containing the local linear regression statistics
            by vial
        vial (dict) : Dictionary of DataFrames sliced by vial, keys are vials and values
            are DataFrames
        '''
        print('---------------> create_df_by_vial: Creates a DataFrame for each vial')
        self.vial,self.result = dict(),dict()


        for i in range(1,self.vials + 1):
            if i == 1: 
                self.vial[i] = self.df_filtered
            else: 
                self.vial[i] = self.df_filtered[self.df_filtered.vial==i]
            self.result[i] = self.local_linear_regression(self.vial[i]).iloc[0].tolist()
            self.result[i] = ['_'.join(self.vial_ID) + '_' + str(i)] + self.result[i]

            self.result[i][1:3] = [int(item) for item in self.result[i][1:3]]
            self.result[i][3:] = [round(item,4) for item in self.result[i][3:]]
        return


    def setting_final_slopes(self,df, pixel_to_cm, save=True):
        '''Taking most linear slope from each vial and saving it to a slopes.csv file'''
        print('---------------> setting_final_slopes: ')
        cols = ['vial_ID','first_frame','last_frame','slope','intercept','r_value','p_value','std_err']
        self.df_slopes = pd.DataFrame.from_dict(df,orient='index',columns = cols)
        if self.convert_to_cm:
            self.df_slopes['slope'] = self.df_slopes['slope'] / int(pixel_to_cm)
            self.df_slopes['slope'] = self.df_slopes['slope'].round(4)

        if save:
            self.df_slopes.to_csv(self.name_nosuffix + '.slopes.csv',index=False)
        return self.df_slopes


    def bin_vials(self,df, vials):
        '''Bin spots into vials. Function takes into account all points along the x-axis, 
        and divides them into specified number of bins based on the min and max
        points in the array.
        ----
        Note: Number of vials is specified in <file>.cfg'''
        print("---------------> bin_vials: assign spots to a vial")
        bin_lines = pd.cut(df.x,vials,include_lowest=True,retbins=True)[1]
        spot_assignments = pd.cut(df.x,bins=bin_lines,labels=range(1,vials+1))
        ## Important if a middle vial disappears
        counts = np.unique(spot_assignments,return_counts = True)
        for v,c in zip(counts[0],counts[1]):
            if c == 0:
                print('Warning: vial',v,'is empty and cannot be evaluated')
        return bin_lines, spot_assignments


    def local_linear_regression(self, df, method = 'max_r'):
        '''Performs a local linear regression, using a user-defined sliding window'''
        print("---------------> local_linear_regression")
        result_list,result = [],pd.DataFrame()
        llr_columns = ['first_frame','last_frame','slope','intercept','r','pval','err']

        for i in range(len(self.image_stack) - self.window):
            start,stop = int(i),int(i+self.window)
            df_window  = df[(df.frame >= start) & (df.frame <= stop)]

            ## Testing if there are enough frames in the slice
#             if len(df_window.frame.unique()) < self.window-2: # Kept getting errors checking to see if there were frames with no spots. Going with groupby function instead
            if df_window.groupby('frame').y.count().min() == 0:
                print('Issue with number of frames with flies detected vs. window size')
                print(i,i+self.window,len(df_window.frame.unique()))
                continue
                        
            ## Performs local linear regression
            try: 
                ## Grouping points by frame
                _frame = df_window.groupby('frame').frame.mean()
                _pos  = df_window.groupby('frame').y.mean()

                ## Performing linear regression on subset and formatting output
                _result = linregress(_frame,_pos)
                _result = [start,stop] + np.hstack(_result).tolist()

                ## If slope is not significantly different from 0, then set slope = 0
                if _result[-2] >= 0.05: _result[2] = 0
            except: _result = [start,stop] + [np.nan,np.nan,np.nan,np.nan,np.nan]

            result_list.append(_result)
        
        result = pd.DataFrame(data=result_list,columns=llr_columns)

        if method == 'max_r':
            result = result[result.r == result.r.max()]
        elif method == 'min_err':
            result = result[result.err == result.err.min()]
        else:
            print("Unrecognized method, chose either 'max_r' to select the window with the greatest R or 'min_err' to select the window with the lowest error")
        return result
        
        
    def step_1(self):
        '''Words'''
        print('-- [ Step 1 ] Cleaning and format image stack')
        y = self.y
        y_max = int(y + self.h)
        x = self.x
        x_max = int(x + self.w)
        stack = self.image_stack
        self.clean_stack = self.crop_n_grayscale(stack,
                         y=y, y_max=y_max,
                         x=x, x_max=x_max)
        print(self.clean_stack.shape)

        self.spot_stack,self.background = self.subtract_background(video_array=self.clean_stack)
#         plt.figure()
#         self.display_images(self.clean_stack,self.background,self.spot_stack,frame=20)
#         plt.savefig(self.name_nosuffix + '.processed.png', dpi=100)
#         plt.close()
        return


    def step_2(self):
        print('-- [ Step 2a ] Detecting flies')
        self.df_big = self.particle_finder(minmass=self.minmass,diameter=self.diameter,
                                            maxsize=self.maxsize, invert=True)
        self.df_big.to_csv(self.name_nosuffix + '.raw.csv', index=None)
        print('                -->',self.path_data.split('/')[-1])

        ## Auto-detecting threshold
        if self.threshold == 'auto':
            self.threshold = self.find_threshold(self.df_big.signal)
        
        self.df_big['True_particle'] = self.df_big.signal.map(lambda x: x >= self.threshold)
        if self.df_big[self.df_big.True_particle].shape[0] == 0:
            print('\n\nExiting program: Threshold limit is too restrictive, no spots left after filtering')
            raise SystemExit
        
        print('-- [ Step 2b ] Detecting flies, filtering and binning into vials')        
        ## Filtering spots and pruning unnecessary columns
        self.df_filtered = self.df_big[self.df_big.True_particle]
        self.df_filtered = self.df_filtered.drop(['ecc','signal','ep','raw_mass','mass','size','True_particle'],axis=1)
    
        ## Binning spots into vials
        self.bin_lines, self.df_filtered['vial'] = self.bin_vials(self.df_filtered,vials = self.vials)
        self.df_filtered = self.df_filtered.sort_values(by=['vial','frame','y','x'])

        print('-- [ Step 2c ] Adding experimental variables to DataFrame\n')
        for item in self.file_details.keys():
            self.df_filtered[item] = np.repeat(self.file_details[item],self.df_filtered.shape[0])
        
        return

    def step_3(self):
        print('-- [ Step 3 ] Visualize spot metrics')

        plt.figure()
        self.view_ROI(border = True,
                        x0 = self.x, x1 = self.x + self.w,
                        y0 = self.y, y1 = self.y + self.h)
        plt.savefig(self.name_nosuffix + '.ROI.png',dpi=100)
        plt.close()
                    
        self.visualize_metrics(self.df_big)
        plt.savefig(self.name_nosuffix + '.spot_check.png',dpi=300)
        plt.close()
        return
        
    def step_4(self):
        print('-- [ Step 4 ] Calculating local linear regressions')
        
        ## Invert the y-axis
        self.df_filtered['y'] = abs(self.df_filtered.y.max() - self.df_filtered.y)
        self.df_filtered['y'] = self.df_filtered.y.round(2)

        ## Save the filtered DataFrame
        self.df_filtered.to_csv(self.name_nosuffix+'.filtered.csv', index=False)
        print('                -->',self.path_filter.split('/')[-1])
        return
        
    def step_6(self,gui=False):
        print('-- [ Step 6 ] Plotting data')
        plt.figure(figsize=(10,8))
        fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(nrows=2, ncols=2)
#         fig.suptitle(self.name)

        ## Finding the frames that flank the most linear portion 
        ##    of the y vs. t curve for all points, not just by vials
        begin = self.local_linear_regression(self.df_filtered).iloc[0].first_frame.astype(int)
        end = begin + self.window

        print("Filtering")
        ## Filtering for true points, then binning into vials
        spots = self.df_big[self.df_big.True_particle]
        self.bin_lines, self.df_big['vial'] = self.bin_vials(self.df_big, self.vials)

        ## Plot: Image of 'begin' frame with spots
        print("Plot 1")
        self.image_plot(df = spots,frame = begin, ax=ax1) # ADAM...issue if only one vial
        ## Plot: Image of 'end' frame with spots
        print("Plot 2")
        self.image_plot(df = spots, ax=ax2, frame = int(str(end)))
        ## Plot: Image of 0th frame with all spots from video
        print("Plot 3")
        self.image_plot(df = spots,ax=ax4, frame = None)
        print("Create DF")                  
        ## Breaking DataFrame into vials
        self.create_df_by_vial()
        print("Plot 4")
        self.loclin_plot(ax=ax3)
        fig.tight_layout()
        plt.savefig(self.name_nosuffix + '.diagnostic.png',dpi=500, transparent = True)
        plt.close()
        print('                --> Saved:',self.path_plot.split('/')[-1])
        
        if gui:
            return ax
        else:
            return
    
    def step_7(self):
        print('-- [ Step 7 ] Setting up slopes file')
        slope_columns = ['vial_ID','first_frame','last_frame','slope','intercept','r_value','p_value','std_err']
        self.df_slopes = pd.DataFrame.from_dict(self.result,orient='index',columns = slope_columns)
        if self.convert_to_cm:
            self.df_slopes['slope'] = self.df_slopes.slope.transform(lambda x: x * (self.pixel_to_cm / self.frame_rate)).round(6)
        
        for item in self.file_details.keys():
            self.df_slopes[item] = np.repeat(self.file_details[item],self.df_slopes.shape[0])
    
        ## Saving slope file
        self.df_slopes.to_csv(self.name_nosuffix + '.slopes.csv',index=False)
        print('                --> Saved: %s \n' % self.path_slope.split('/')[-1])
        return
        
    def image_plot(self,df,frame=None,ax=None,ylim=[0,1000]):
        ## Can probably replace this with the image plot function in the test_parameters function below
        ## Assigning plotting parameters based on frame
        try: frame = int(frame)
        except: frame = None

        ## Determine which frame to plot
        if frame == None:
            frame = 0
            alpha = 0.01
            ax.set_title('All x,y-points throughout video')
        elif type(frame) == int:
            df = df[(df.frame == frame)]
            alpha = .25
            ax.set_title("Frame: %s" % frame)
        else:
            print("Chose a frame value in integer form, or 'None'")

        ## Issue with plotting if last frame in stack
        if frame == self.n_frames:
            image = self.clean_stack[frame-1]
        else:
            image = self.clean_stack[frame]

        ## Plotting image
        ax.imshow(image,cmap=cm.Greys_r,origin='upper')
        
        ## Plotting vertical bin lines
        ax.vlines(self.bin_lines,0,image.shape[0],alpha = .3)  

        ## Plotting point overlay
        df['vial'] = self.bin_vials(df,self.vials)[1] ## Issue if only one vial?
        df = df.sort_values(by='vial')
        if self.vials > 1:
            ax.scatter(df.x,df.y,s=30,alpha=alpha,
                    c = df['vial'], cmap=self.discrete_cmap(vials = self.vials))
        else:
            ax.scatter(df.x,df.y,s=30,alpha=alpha,
                    color = self.color_list[0])
       
        ## Getting rid of axis labels
        ax.xaxis.set_visible(False)
        ax.yaxis.set_visible(False)
        return ax
    
    def loclin_plot(self,ax = None):
        print("---------------> loclin_plot")
        def two_plot(df,color,label,first, last, ax=None):
            print("---------------> two_plot")
            
            x = df.groupby('frame').frame.mean()
            y = df.groupby('frame').y.mean()
            ax.plot(x,y, alpha = .35, color = color,label='') 

            df = df[(df.frame >= first) & (df.frame <= last)]
            x = df.groupby('frame').frame.mean()
            y = df.groupby('frame').y.mean()
            ax.plot(x,y,color = color,label = label)
            return
        
        if len(self.result) > 1:
            for V in range(1,self.vials+1):
                l = 'Vial '+str(V)
                c = self.color_list[V-1]
                _details = self.result[V]
                frames = _details[1],_details[2]
                two_plot(self.vial[V],color = c,label = l,
                         first = _details[1],last = _details[2],ax=ax)
        else:
            for V in range(1,self.vials+1):
                l = 'Vial 1'
                c = self.color_list[V-1]
                _details = self.result[V]
                frames = _details[1],_details[2]
                two_plot(self.vial[V],color = c,label = l,
                         first = _details[1],last = _details[2],ax=ax)

        ax.legend(loc=2, frameon=False, fontsize='x-small')

        label = 'pixels'
        if self.convert_to_cm: label = 'cm'
        labels = ['Cohort climbing kinematics', 'Frames','Mean y-position (%s)' % label]
        ax.set(title=labels[0],xlabel = labels[1],ylabel=labels[2])
        return

    def discrete_cmap(self, vials = None):
        ## Modified from: https://gist.github.com/jakevdp/91077b0cae40f8f8244a
        '''Creates a list of colors from a color map, given a specified number of vials'''
        if vials == None:
            vials = 1
            base_cmap = 'tab10'
        elif self.vials <= 10: base_cmap = 'jet'
        elif self.vials <= 20: base_cmap = 'tab20'
        elif self.vials > 20: base_cmap = 'jet'

        base = plt.cm.get_cmap(base_cmap)
        
        ## If there is an odd number of vials, adjust for having a too-light 'middle' color
        if vials%2 and vials > 1:
            self.color_list = base(np.linspace(0, 1, vials+1))
            row_to_drop = len(self.color_list)//2
            self.color_list = np.delete(self.color_list, row_to_drop,axis = 0)
        else:
            self.color_list = base(np.linspace(0, 1, vials))
        
        self.cmap_name = base.name + str(self.vials)
        return LinearSegmentedColormap.from_list(self.cmap_name, self.color_list, vials)

    
    #####    #####    #####    #####    #####    #####    #####    #####    #####
    ## Commands used only in GUI
    #####    #####    #####    #####    #####    #####    #####    #####    #####
    def parameter_testing(self, variables, axes):
        print('-------> parameter_testing')
        self.load_for_gui(variables)
        self.step_1() # Load video
        self.step_2() # Crop video
        self.step_3() # Visualize other spots
        
        spots = self.df_big
        SPOTS = spots[spots.True_particle]
        bin_lines,SPOTS['vial'] = self.bin_vials(SPOTS,self.vials)
        self.discrete_cmap(vials=self.vials)
        SPOTS.loc[:,'color'] = SPOTS.vial.map(dict(zip(range(1,self.vials+1),self.color_list)))
        print(SPOTS.head())
        frame_0 = self.frame_0
        frame_n = self.frame_n
        bins=20
        
        ## Setting plot for background image
        axes[0].set_title("Background Image")
        axes[0].imshow(self.background,cmap=cm.Greys_r)
        axes[0].scatter([0,self.w],[0,self.h],alpha=0,marker='.')

        ## Setting plots for scatterplot overlay
        frame=40
        axes[1].set_title('Frame:'+str(frame_0))
        axes[1].imshow(self.clean_stack[frame_0], cmap = cm.Greys_r)
        axes[1].scatter(spots[(spots.frame==frame_0) & (~spots.True_particle)].x,
                        spots[(spots.frame==frame_0) & (~spots.True_particle)].y, 
                        color='w',marker = 'x', alpha = .5)
        a = axes[1].scatter(SPOTS[SPOTS.frame==frame_0].x,
                            SPOTS[SPOTS.frame==frame_0].y, 
                            color = SPOTS[SPOTS.frame==frame_0].color,
                            marker ='o',alpha = .8)
        a.set_facecolor('none')
        axes[1].vlines(bin_lines,0,spots.y.max(),color='w')
        
        ## Fly counts
        axes[5].plot(SPOTS.groupby('frame').frame.mean(),
                     SPOTS.groupby('frame').frame.count(), 
                     label = 'Fly count', color = 'g',alpha = .5)
        axes[5].hlines(np.median(SPOTS.groupby('frame').frame.count()),
                       spots.frame.min(),spots.frame.max(),
                       linestyle = '--',alpha = .5, 
                       color = 'gray', label = 'Median no. flies')
        axes[5].set(title = 'Flies per frame',
                        ylabel='Flies past threshold',
                        xlabel='Frame') 
        axes[5].legend(frameon=False, fontsize = 'small')

        ## Mass histogram
        axes[3].set_title('Mass Distribution')
        axes[3].hist(spots.mass,bins = bins)
        y_max = np.histogram(spots.mass,bins=bins)[0].max()
        axes[3].vlines(self.threshold,0,y_max)

        ## Signal histogram
        axes[4].set_title('Signal Distribution')
        axes[4].hist(spots.signal,bins = bins)
        y_max = np.histogram(spots.signal,bins=bins)[0].max()
        axes[4].vlines(self.threshold,0,y_max)

        ## Setting plots for local linear regression
        df = SPOTS.sort_values(by='frame')
        df['y'] = self.invert_Y(df)
        for V in range(1,self.vials + 1):
            label = 'Vial '+str(V)
            color = self.color_list[V-1]
            _df = df[df.vial == V]
            
            begin = self.local_linear_regression(_df).iloc[0].first_frame.astype(int)
            end = begin + self.window        
            frames = begin,end
            
            axes[2].plot(_df.groupby('frame').frame.mean(),
               _df.groupby('frame').y.mean(),alpha = .35, color = color,label='') 
            _df = _df[(_df.frame >= begin) & (_df.frame <= end)]

            axes[2].plot(_df.groupby('frame').frame.mean(),
               _df.groupby('frame').y.mean(),color = color, label = label)

        labels = ['Mean vertical position over time','Mean vertical position','Frame']
        axes[2].set(title = labels[0], ylabel=labels[1],xlabel=labels[2]) 
        axes[2].legend(frameon=False, fontsize='x-small')            
        return
#         


#!/usr/bin/env python
# -*- coding: utf-8 -*- 

## Importing external packages
import os
import detector
import sys
from time import time
from numpy import sort, repeat
from matplotlib.cm import Greys_r
import pandas as pd
import matplotlib.pyplot as plt


def load_variables(config_file):
    ## Finding the path_project
    if os.path.isfile(config_file):
        pass
    else:
        print('\n\nExiting program. Invalid configuration file')
        raise SystemExit ## Issue when running GUI, terminates this script, but not the GUI

    with open(config_file,'r') as f:
        print('## Reading in configuration file:', config_file)
        variables = f.read().split('\n')
    f.close()

    ## Importing configuration file variables
    variables = [item for item in variables if ~item.startswith(' ') or item.startswith('#')]
    return variables

def check_arguments():
    ## I want to add arguments for which videos things to processes, so the GUI doesn't do all, or someone can specify 'how' they want it done: 1, not yet processed (undone), or all
    args = sys.argv[1:]
    if len(args) == 0 or len(args) > 2:
        print('\n\nExiting program. Too many or too few arguments -- required argument 1 is .cfg file path and optional argument 2 is a boolean to concatenate all slope files at the end')
        raise SystemExit ## Issue when running GUI, terminates this script, but not the GUI

    if os.path.isfile(args[0]) and args[0].endswith('.cfg'):
        config_file = args[0]
    else:
        print('\n\nExiting program. Invalid configuration file')
        raise SystemExit ## Issue when running GUI, terminates this script, but not the GUI
    try:
        if type(args[1]) == bool:
            pass
    except:
        args.append(False)
    return args


class free_climber(object):
    def __init__(self, cfg_file):
        self.cfg_file = cfg_file
        
    def load_parameters(self):
        var_list = []
        with open(self.cfg_file,'r') as f:
            variables = f.read().split('\n')
        f.close()
        variables = [item for item in variables if not item.startswith('#') or not item.startswith(' ') or not item.startswith('\n')]
        for item in variables:
            try:
                exec('self.'+item)
            except:
                pass
        return
        
    def file_walker(self,folder = None,endswith=None):
        '''
        From a specified parent folder, find all child files with the specified suffix
        '''
        _lst=[]
        for root, dirs, files in os.walk(folder):
            for name in files:
                if name.endswith(endswith):
                    _lst.append((os.path.join(root, name)))
        return sorted(_lst)

    def undone(self,folder=None):
        '''
        Used in conjunction with file_walker, this function finds all the files that have
        yet to be processed.
        '''
        _lst1,_lst2 = [],[]
        for root,dirs,files in os.walk(folder):
            for name in files:
                if name.endswith(self.file_suffix):
                    _lst1.append(os.path.join(root, name[:-5]))
                if name.endswith('slopes.csv'):
                    _lst2.append(os.path.join(root, name[:-4]))

        _lst1,_lst2 = set(_lst1),set(_lst2)

        todo = list(_lst1.difference(_lst2))
        todo = [item+'.'+self.file_suffix for item in todo]
        return sorted(todo)

    def timer(self, time_begin):
        time_elapsed = time()-time_begin
        time_elapsed = str(int(time_elapsed//60))+" min : " + str(int(time_elapsed%60))+" seconds"
        print("Time elapsed: ",time_elapsed)
        print('='*72)
        return

    def setup(self):
        self.file_list = self.file_walker(self.path_project,self.file_suffix)
        self.todo = self.undone(self.path_project)
        self.count = 0
        self.first_run = True
        self.print_new_project()
        return
        
    def print_new_video(self,name):
        print('='*72)
        line_to_print = '== [%s || %s] ' % (self.count,len(self.todo)), self.name
        line_to_print = ('').join(line_to_print)
        print(('').join(line_to_print), '=' * (71 - len(line_to_print)))
        print('='*72)
        return

    def print_new_project(self):
        print("##\n## Analyzing videos in: %s \n##" % self.path_project )
        print("## Of the %s videos in path_project, %s remain unprocessed\n##\n##" % (str(len(self.file_list)), str(len(self.todo))) )
        return

    def process(self,File,variables):
        self.print_new_video(self.name)

        d = detector.detector(File)
        d.step_1()#File)#,variables)
        d.step_2()    
#         d.step_3()
        d.step_4()
        ## -- [ Step 5 ] - Calculating local linear regressions
#         d.step_5()

        ## -- [ Step 6 ] - Plotting data
        d.step_6()

        ## -- [ Step 7 ] - Setting up slopes file
        d.step_7()
        
        self.first_run = False
        return

    def concat_slopes(self):
        print("\nFinal step: Concatenating slope files")
        slope_files = self.file_walker(self.path_project,endswith='.slopes.csv')
        to_concat = [slope_file for slope_file in slope_files]
        print("    - Concatenating",len(to_concat),"files")
        self.slopes = pd.concat([pd.read_csv(item) for item in to_concat])
        self.path_result = self.path_project+'/results.csv'
        self.slopes.to_csv(self.path_result,index=False)
        return
    
    def check_variables(self):
        ## Making sure diameter is an odd number
        if int(self.diameter) % 2 == False:
            self.diameter = int(self.diameter) + 1
            
        ## Should probably check other variables at some point...
        
        return

def print_opening():
    ## First few lines when running script
    print('\n')
    print('#' * 72)
    print('## Vial Detector 1.0','#'*51)
    print('#' * 72)
    return         

def main(cfg_file):
    print_opening()
    variables = load_variables(cfg_file)
    vd = free_climber(cfg_file = cfg_file)
    vd.cfg_file = cfg_file
    vd.load_parameters()
    vd.setup()
    vd.check_variables()
#     for File in vd.file_list:
    for File in vd.todo:
        t0 = time()
        vd.count += 1
        vd.name = os.path.split(File)[-1]
        vd.process(File,variables)
        vd.timer(t0)

    if args[1]:
        vd.concat_slopes()
    
    return
        
if __name__ == '__main__':
    args = check_arguments()
    cfg_file = args[0]
    main(cfg_file)
    exit()


#!/usr/bin/env python

import argparse
import os

def read_custom(file):
    '''Reads in process file (.prc) containing individual video file paths to process.
    ----
    Inputs:
      file (str): Path to custom file (.prc)
      suffix (str): Video file suffix read in from configuration (.cfg) file
    ----
    Returns:
      _lines (list): List of files read from custom file (.prc)
    '''
    ## Read in lines from custom file
    with open(file) as f:
        lines = [line.rstrip() for line in f]
    f.close()
    
    ## Check files are real
    _lines = [vid for vid in lines if os.path.isfile(vid)]
    print('----> %s of %s specified files have valid paths' % (len(lines),len(lines)))
    return _lines
    
    
def define_argument_parser():
    '''Defines arguments to be parsed, via argparse module. 
    Given a method argument, 'gui' or 'batch', different downstream arguments are required.
    
    For 'gui', an optional 'file' argument specifies which video to being with, though
    if not specified the program will open a browser to select the file. 
    
    For 'batch', the '--file' flag is required and points to a configuration file (.cfg)
    created from the GUI. Files to process are specified for all (--process-all), 
    undone (--process_undone), or a custom list (--process_custom) of videos. If a custom
    list, include the path to a process file (.prc) containing all files to process, 
    each on a separate line. 
    
    Additional flags following '--batch' for concatenating ('--concat') all results 
    files and performing a repeated measures ANOVA statistics ('--stats') for longitudinal
    studies can also be specified.
    ----
    Inputs:
      None
    ----
    Returns:
      args (list): list of arguments passed to program
    '''
    parser = argparse.ArgumentParser(prog='FreeClimber',
                                    description='Calculates the climbing velocity of flies in a negative geotaxis assay',
                                    usage='%(prog)s [options] path',
                                    epilog='For documentation and a tutorial, see https://github.com/adamspierer/FreeClimber',
                                    allow_abbrev=False)

    ## Analysis platform
    group_method = parser.add_mutually_exclusive_group(required=True)
    group_method.add_argument('-b', '--batch', 
                                action='store_true', 
                                help="Run from command line")
    group_method.add_argument('-g', '--gui', 
                                action='store_true', 
                                help="Run from GUI")

    ## Specifying file (video file for gui or configuration file for batch)
    parser.add_argument('--file', 
                        type=str, 
                        required=False, 
                        help="Path to file (.cfg file if -b/--batch; (optional) video file if '-g/--gui')")

    ## Batch processing: specifying which files to process
    parser.add_argument('--process_all', 
                        default=True, 
                        action='store_true', 
                        help="With 'batch', process all files")
    parser.add_argument('--process_undone', 
                        default=False,
                        action='store_true', 
                        help="With 'batch', process previously unprocessed files")
    parser.add_argument('--process_custom', 
                        default=False, 
                        type=str,
                        help="With 'batch', process custom list of files. Include file path as argument")

    ## Batch processing: additional arguments
    parser.add_argument('--concat', 
                        required=False, 
                        default=True, 
                        action='store_true',
                        help="With 'batch', concatenate results into single file")
    parser.add_argument('--stats', 
                        required=False, 
                        default=False, 
                        action='store_true',
                        help="With 'batch', performs a repeated measures ANOVA. See documentation for caveats.")

    ## Final parser specifications
    parser.version = '1.0' ## Program version
    parser.add_argument('-v','--version', 
                        action='version')
    args = parser.parse_args()
    return args


## Check input arguments
def startup():
    '''Function prints top line and runs argument parsing function.
    ----
    Inputs:
      None
    ----
    Returns:
      args (list): list of arguments passed to program
    '''
    print('#'*80+'\n'+'Running FreeClimber\n')
    args = define_argument_parser()
    return args


def check_args(args):
    '''Checks arguments passed to parser.
    Inputs:
      args (list): Arguments passed to parsed
    ----
    Returns:
      None
    '''
    print('check_args')
    if args.gui:
        ## Confirm file path is valid
        if args.file != None and os.path.isfile(args.file):
            print('--> Video file: ',args.file)
        elif args.file != None and ~os.path.isfile(args.file):
            print('--> Exiting program: Invalid file path. Make sure video file for GUI is properly specified.')
            print('!!', args.file)
            raise SystemExit
        else:
            print('--> No video file specified, select from Browser')
            args.file = ''

    else:
        print('Batch')
        ## Checking configuration file
        if args.file == None:
            print('--> Exiting program: Must include path to configuration (.cfg) file')
            raise SystemExit
        elif os.path.isfile(args.file) and args.file.endswith('.cfg'):
            pass
        else:
            print('--> Exiting program: Invalid configuration file, make sure proper file with ".cfg" is specified.')
            print('!!', args.file)
            raise SystemExit
    
        ## Checking which files to process
        if args.process_all == None:
            print('--> Processing all video files')
        elif args.process_undone:
            print('--> Processing undone video files')
        elif args.process_custom != None:
            print('--> Processing a list of files in a specified file a custom file list')
            if os.path.isfile(args.process_custom) and args.process_custom.endswith('.prc'):
                process_custom = read_custom(args.process_custom)
            else:
                print("--> Exiting program: Invalid file path or format, make sure proper file is specified.")
                print('!!', args.process_custom)
                raise SystemExit
    return True


def main():
    '''Running main function'''
    args = startup()
    check_args(args)
    if args.gui:
        print('COMMAND: pythonw ./FreeClimber_gui.py %s' % args.file)
    if args.batch:
        print('COMMAND: python ./FreeClimber_batch.py %s' % args.file)
    print(args)
    print('Done!')
    return


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

# Get the long description from the relevant file
with open('README.md','r') as f:
    long_description = f.read()

setup(name='FreeClimber',
      version='0.3',
      description='A background-adjusted, particle detector used to quantify climbing performance for groups of Drosophila in a negative geotaxis assay',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://github.com/adamspierer/FreeClimber',
      
      author='Adam Spierer',
      author_email='anspierer+Github_setup_py@gmail.com',
      license='MIT',

      classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3.7'],

      keywords='Drosophila climbing negative geotaxis background subtraction particle detection local linear regression',

      packages=find_packages(),
#       packages=['FreeClimber'],
      install_requires=['ffmpeg==1.4',"argparse==1.1",
                        'pandas','numpy','scipy','pip','matplotlib==3.1.3',
                        'wxPython==4.0.4','trackpy==0.4.2'],
      zip_safe=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*- 
#Boa:Frame:DectFrame

import wx
import os
import sys
import time
import numpy as np
import pandas as pd

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.patches import Rectangle
import matplotlib.cm as cm
import matplotlib.pyplot as plt

from detector import detector
import vial_detector_main

print('')

class DectFrame(wx.Frame):    
    def __init__(self, parent, file_path):
        print('-------->  __init__')
        self._init_controls(parent)
        self.box_sizer.Add(self.panel1, 0, wx.EXPAND)

        self.pressed=False
        ## Initialize GUI plot
        self.figure = Figure()
        self.canvas = FigureCanvas(self, -1, self.figure)
        self.box_sizer.Add(self.canvas, 1, wx.EXPAND)

        ## Initialize bottom text bar
        self.box_sizer.Add(self.status_bar, 0, border=0, flag=0)
        self.status_bar.SetStatusText('Ready', 0)
        rect = self.status_bar.GetFieldRect(0)
    

        ## Set up names
        self.video_file = file_path
        
        ## Check input file is valid
        if not os.path.isfile(file_path):
            openFileDialog = wx.FileDialog(self, "Open Video file", "", "",
                                       "Video files (*.*)|*.*", 
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

            if openFileDialog.ShowModal() == wx.ID_CANCEL:
                print('\n\nExiting Program. Must select a video from box or enter path into command line\n\n')
                raise SystemExit

            self.video_file = openFileDialog.GetPath()        
            file_path = self.video_file
        ## Passing individual inputs from GUI to a list
        self.update_names()
        self.input_names = ['x','y','w','h','frame_0','frame_n','blank_0','blank_n',
        'threshold','diameter','minmass','maxsize','vials','window','pixel_to_cm','frame_rate',
        'vvars','naming_convention','path_project','file_suffix','convert_to_cm']
        self.parameter_names = ['self.input_'+item for item in self.input_names]
        self.input_values = [self.input_x,self.input_y,self.input_w,self.input_h,
                             self.input_frame_0,self.input_frame_n,self.input_blank_0,
                             self.input_blank_n,self.input_threshold,self.input_diameter,
                             self.input_minmass,self.input_maxsize,self.input_vials,
                             self.input_window,self.input_pixel_to_cm,self.input_frame_rate,
                             self.input_vvars,self.input_naming_convention,self.input_path_project,
                             self.input_file_suffix,self.input_convert_to_cm]

        ## Enable all buttons
        button_list = ['browse_video','reload_video','test_parameters','store_parameters','run_analysis']
        for button in button_list:
            exec('self.button_' + button + '.Enable(True)')

        self.load_video()
        print('End of init')
        return

    def update_variables(self):
        print('---------> update_variables')
        variables = []
        
#         self.input_convert_to_cm = self.input_convert_to_cm.GetValue()
        for item,jtem in zip(self.input_names[:17],self.input_values[:17]):
            phrase = str(item + '='+ jtem.GetValue())
            print('    '+phrase)
            variables.append(phrase)
        for item,jtem in zip(self.input_names[17:19],self.input_values[17:19]):
            phrase = str(item + '="'+ jtem.GetValue()+'"')
            print('    '+phrase)
            variables.append(phrase)
        for item,jtem in zip(self.input_names[19:20],self.input_values[19:20]):
            phrase = str(item + '="'+ str(jtem)+'"')
            print(phrase)
            variables.append(phrase)
        for item,jtem in zip(self.input_names[20:],self.input_values[20:]):
            phrase = str(item + '=%s' % str(jtem.GetValue()))
            print('    '+phrase)
            variables.append(phrase)

        return variables

    def load_video(self):
        print('--------> load_video')
        #Load the video, set Cursor to be busy
        self.figure.clear()
        self.axes   = [self.figure.add_subplot(111), ]
        
        ## Confirm file is a path, or folder has specified suffix
        status = self.check_specified_video()
        if status:
            print("Loading:", self.video_file)
#             self.input_frame_rate.SetValue('25')
            self.update_names()
            for item in self.input_values[:4]:
                item.SetEditable(True)
                item.Enable(True)
            self.checkBox_fixed_ROI.Enable(True)
            self.input_convert_to_cm.Enable(True)
            wx.BeginBusyCursor()
            try:
                variables = self.update_variables()
                self.detector = detector(self.video_file,
                                        gui=True,
                                        variables = variables)
#                 self.input_frame_rate.SetValue(str(self.detector.frame_rate))
            
                self.axes[0].imshow(self.detector.image_stack[0])
                self.figure.canvas.draw()
            finally:
                wx.EndBusyCursor()

            self.canvas.Bind(wx.EVT_ENTER_WINDOW, self.ChangeCursor)
            self.canvas.mpl_connect('button_press_event', self.draw_rectangle)
            self.canvas.mpl_connect('button_release_event', self.on_release)
            self.canvas.mpl_connect('motion_notify_event', self.on_motion)
            self.rect = Rectangle((0,0), 1, 1, fill=False, ec='r')
            self.axes[0].add_patch(self.rect)


            ## Auto-set GUI parameters form video
#             self.input_frame_rate.SetValue(str(self.input_frame_rate))
            self.input_blank_0.SetValue('0')
            self.input_blank_n.SetValue(str(self.detector.n_frames))
            self.input_frame_0.SetValue('0')
            self.input_path_project.SetValue(self.folder)
            self.input_naming_convention.SetValue(self.name)
            self.input_vvars.SetValue(str(len(self.input_naming_convention.GetValue().split('_'))))
            ## Display the 0th and frame corresponding with (most likely) t = 2 seconds
            try:
                self.input_frame_rate = int(self.input_frame_rate.GetValue())
            except:
                pass
            if self.detector.n_frames < self.input_frame_rate*2:            
                self.input_frame_n.SetValue(str(self.detector.n_frames))
            else:
                self.input_frame_n.SetValue(str(self.input_frame_rate*2))
            
            ## Try to make the local linear regression window size 2 seconds, but if not then 35% of the frames in the video
            if self.detector.n_frames < self.input_frame_rate*2:
                self.input_window.SetValue(str(int(len(self.detector.image_stack) * .35)))                
            else:
                self.input_window.SetValue(str(int(self.input_frame_rate)*2))

            ## Enable Test parameter button if disabled from prior testing
            self.button_test_parameters.Enable(True)
        
            self.x0 = 0
            self.y0 = 0
            self.x1 = self.detector.width
            self.y1 = self.detector.height
        
            self.update_ROIdisp()
            self.canvas.draw()
        else:
            return

    def update_names(self):
        print('--------> update_names')
        self.text_video_path.SetLabelText(self.video_file)
        self.folder, self.name        = os.path.split(self.video_file)
        print(self.name)
        self.name,   self.input_file_suffix = self.name.split('.')

        self.name_noext  = os.path.join(self.folder,self.name)
        self.path_data   = self.name_noext + '.raw.csv'
        self.path_filter = self.name_noext + '.filter.csv'
        self.path_plot   = self.name_noext + '.diag.png'
        self.path_slope  = self.name_noext + '.slopes.csv'
        
        if self.input_path_project == '':
            self.input_path_project = self.folder


    def check_specified_video(self):
        print('--------> check_specified_video')
        if os.path.isfile(self.video_file):
            self.button_reload_video.Enable(True)
            self.update_names()
            
#             self.button_reload_video.Enable(False)
            self.button_test_parameters.Enable(True)
            
            self.input_file_suffix = '.' + self.video_file.split('/')[-1].split('.')[-1]
            return True

        elif file_path == None:
            self.button_browse_video.Enable(True)
            return False

        else:
            self.video_file = "Invalid file. Please change the file path"
            self.button_browse_video.Enable(True)
            return False


    def ChangeCursor(self, event):
        '''Change cursor into crosshair type when enter the plot area'''
        self.canvas.SetCursor(wx.Cursor(wx.CURSOR_CROSS))


    def draw_rectangle(self, event):
        '''Draw ROI rectangle'''
        self.pressed = True
        if self.checkBox_fixed_ROI.Enabled:
            try:
                self.x0 = int(event.xdata)
                self.y0 = int(event.ydata)
                if self.checkBox_fixed_ROI.GetValue():
                    self.x1 = self.x0 + int(eval(self.input_w.GetValue()))
                    self.y1 = self.y0 + int(eval(self.input_h.GetValue()))
                    self.rect.set_width(self.x1 - self.x0)
                    self.rect.set_height(self.y1 - self.y0)
                    self.rect.set_xy((self.x0, self.y0))
                    self.canvas.draw()
                self.input_x.SetValue(str(self.x0))
                self.input_y.SetValue(str(self.y0))
                self.input_h.SetValue(str(self.rect.get_height()))
                self.input_w.SetValue(str(self.rect.get_width()))
            except:
                pass


    def on_release(self, event):
        '''When mouse is on plot and button is released, redraw ROI rectangle, update ROI values'''
        self.pressed = False
        if self.checkBox_fixed_ROI.Enabled:
            if self.checkBox_fixed_ROI.GetValue():
                pass
            else:
                self.redraw_rect(event)
            self.update_ROIdisp()


    def on_motion(self, event):
        '''If the mouse is on plot and if the mouse button is pressed, redraw ROI rectangle'''
        if self.pressed & self.checkBox_fixed_ROI.Enabled & (not self.checkBox_fixed_ROI.GetValue()):
            # redraw the rect
            self.redraw_rect(event)
            self.update_ROIdisp()


    def redraw_rect(self, event):
        '''Draw the ROI rectangle overlay'''
        try:
            x1 = int(event.xdata)
            y1 = int(event.ydata)
            if any([self.x1!=x1, self.y1!=y1]):                
                self.x1 = x1
                self.y1 = y1
                self.rect.set_xy((self.x0, self.y0))
                self.rect.set_width(self.x1 - self.x0)
                self.rect.set_height(self.y1 - self.y0)
                
                self.canvas.draw()
            else:
                pass
        except:
            pass


    def update_ROIdisp(self):
        '''Update ROI display in UI'''
        self.input_x.SetValue(str(self.x0))
        self.input_y.SetValue(str(self.y0))
        self.input_h.SetValue(str(int(self.y1) - int(self.y0)))
        self.input_w.SetValue(str(int(self.x1) - int(self.x0)))
       

    def OnButton_runAnaButton(self, event):
        '''Analysis the whole video, output the result pandas.dataframe and analysis config file
        in the same folder (as the video file).
        Plot diagnostic plots.
        '''
        print('-------->  OnButton_runAnaButton')
        variables = self.update_variables()
        time_begin = time.time()
        self.status_bar.SetStatusText("Running analysis...",0)
        self.button_run_analysis.Enable(False)
        self.save_parameter()

        self.figure.clear()
        self.axes = self.figure.add_subplot(111)
        wx.BeginBusyCursor()

        ## Running the main program
        try:  
#             free_climber_main.free_climber.process(File = self.video_file,variables=variables)
#             raise SystemExit
            os.system('python ./vial_detector_main.py %s False' % (self.path_parameters)) ## HELP
        finally:
            self.status_bar.SetStatusText("Analysis complete. Reload video to test new parameters, try a different video, or batch process all videos using command line",0)
            wx.EndBusyCursor()

        ## Plotting the resulting diagnostic plot
        ##   --> Would rather this be plot directly into the GUI, rather than loading
        ##       a picture to the GUI that has poor resolution
        image = plt.imread(self.path_plot)
        self.axes.imshow(image)
        self.axes.spines['top'].set_visible(False)
        self.axes.spines['right'].set_visible(False)
        self.axes.xaxis.set_visible(False)
        self.axes.yaxis.set_visible(False)
        self.figure.canvas.draw()       

        self.button_reload_video.Enable(True)
        self.button_store_parameters.Enable(True)
        self.button_run_analysis.Enable(True)
        self.button_test_parameters.Enable(True)
        return

    def OnButton_testParButton(self, event):
        '''plot the decection result and compare two frames, an early and a late Frame
        So the parameter can be fine tuned.'''
        print('-------->  OnButton_testParButton')

        #Prep the parameters
        variables = self.update_variables()
        self.checkBox_fixed_ROI.Enable(False)

        ## Set up figure for plots
        self.figure.clear()
        self.axes = [self.figure.add_subplot(231),
                     self.figure.add_subplot(232),
                     self.figure.add_subplot(233),
                     self.figure.add_subplot(234),
                     self.figure.add_subplot(235),
                     self.figure.add_subplot(236)]

        wx.BeginBusyCursor()
        try:
            self.detector.parameter_testing(variables, self.axes)
        finally:
            wx.EndBusyCursor()
        
        self.figure.tight_layout()
        self.figure.canvas.draw()

        #Once the ROI is defined, disable further changes
        self.button_reload_video.Enable(True)
        self.button_store_parameters.Enable(True)
        self.button_run_analysis.Enable(True)
        self.button_test_parameters.Enable(False)


    def OnButton_strParButton(self, event):
        print('-------->  OnButton_strParButton')
        self.save_parameter()

    def set_config_file(self):
        '''DocString'''
        ## Figure out where to save configuration file
        if os.path.isdir(self.input_path_project):
            if not self.input_path_project.endswith('/'):
                self.input_path_project = self.input_path_project + '/'
            self.path_parameters = self.input_path_project + self.name + '.cfg'
        else:
            self.path_parameters = self.path_noext+'.cfg'
        return self.path_parameters

    def save_parameter(self):
        print('-------->  save_parameter')
        '''
        Save parameters as python list.
        New parameter sets appended to the config file.
        Each parameter sets come with a comment line, contain the datetime of analysis
        '''

        variables = self.update_variables()
        try: self.input_path_project = self.input_path_project.GetValue()
        except: pass

        self.path_parameters = self.set_config_file()

        ## Printing output to configuration file
        print('Saving parameters to:', self.path_parameters)
        with open(self.path_parameters, 'w') as f:
            print('## Vial Detector ##', file=f)
        f.close()
        with open(self.path_parameters,'a') as f:
            print('## Generated from file:'+self.video_file,file = f)
            print('##     @ ' + str(time.ctime()), file = f)
            print('##',file = f)
            print('## Analysis parameters:',file = f)
            for item in variables:
                print(item,file = f)
        f.close()
        print("Configuration settings saved")
        return

    def OnButton_Browse(self, event):
        print('-------->  OnButton_Browse')
#         self.input_frame_rate.SetValue('1')
        openFileDialog = wx.FileDialog(self, "Open Video file", "", "",
                                       "Video files (*.*)|*.*", 
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        if openFileDialog.ShowModal() == wx.ID_CANCEL:
            pass
        else:
            self.video_file = openFileDialog.GetPath()
            self.update_names()
            self.load_video()
        self.figure.clear()
        return


    def OnButton_LoadVideo(self,event):
        print('-------->  OnButton_LoadVideo')
        self.load_video()
        return

        
    def _init_sizers(self):
        print('-------->  _init_sizers')
        # Generated method, do not edit
        self.box_sizer = wx.BoxSizer(orient=wx.VERTICAL)
        self.SetSizer(self.box_sizer)

    def _init_controls(self, prnt):
        print('-------->  _init_controls')
        # Generated method, do not edit
        wx.Frame.__init__(self, id=wxID_text_title, name='', parent=prnt,
              pos=wx.Point(100, 30), size=wx.Size(950, 759),
              style=wx.DEFAULT_FRAME_STYLE,
              title='Background adjusted particle detection')
        self.SetClientSize(wx.Size(950, 737))

        ######
        ## Inputs for ROI Rectangle
        self.panel1 = wx.Panel(id=wxID_panel_1, name='panel1',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(950, 231),
              style=wx.TAB_TRAVERSAL)
       
        ## Step 1 boxes
        self.text_step_1a = wx.StaticText(id=wxID_text_step_1a,
              label=u'Step 1a: Find video', name='text_step_1a', parent=self.panel1, 
              pos=wx.Point(col1,10),
              size=wx.Size(box_dimensions),style = wx.ALIGN_CENTER)
                      
        ## Browse
        self.button_browse_video = wx.Button(id=wxID_browse_video,
              label=u'Browse...', name=u'button_browse_video', parent=self.panel1,
              pos=wx.Point(col1, 30), size=wx.Size(box_dimensions), style=0)
        self.button_browse_video.Bind(wx.EVT_BUTTON, self.OnButton_Browse,
              id=wxID_browse_video)
        
        self.text_step_1b = wx.StaticText(id=wxID_text_step_1b,
              label=u'Step 1b: Define options', name='text_step_1b', parent=self.panel1, 
              pos=wx.Point(col1,65),
              size=wx.Size(box_dimensions),style = wx.ALIGN_CENTER)

        ## Pixel to cm
        self.text_pixel_to_cm = wx.StaticText(id=wxID_text_pixel_to_cm,
              label=u"Pixels / cm:", name='text_pixel_to_cm',
              parent=self.panel1, pos=wx.Point(col1, 115), size=wx.Size(medium_box_dimensions),
              style=0)
        self.input_pixel_to_cm = wx.TextCtrl(id=wxID_input_pixel_to_cm,
              name=u'input_pixel_to_cm', parent=self.panel1, pos=wx.Point(col1 + 95, 115), 
              size=wx.Size(medium_box_dimensions), style=0, value=u"1")
        
        ## Frame Rate    
        self.text_frame_rate = wx.StaticText(id=wxID_frame_rate,
              label=u'Frames / sec:', name='text_frame_rate',
              parent=self.panel1, pos=wx.Point(col1,85), size=wx.Size(box_dimensions),
              style=0)
        self.input_frame_rate = wx.TextCtrl(id=wxID_frame_rate,
              name=u'input_frame_rate', parent=self.panel1, pos=wx.Point(col1 + 95,85),
               size=wx.Size(medium_box_dimensions), style=0, value='25')
        
        ## Check box to convert final slope to cm
        self.input_convert_to_cm = wx.CheckBox(id=wxID_input_convert_to_cm,
              label=u'Convert to cm', name=u'input_convert_to_cm',
              parent=self.panel1, pos=wx.Point(col1, 145), size=wx.Size(250,22),
              style=0)

        ## Step 2 boxes

        self.text_step_2 = wx.StaticText(id=wxID_text_step_2,
              label=u'Step 2: Select ROI', name='text_step_2', parent=self.panel1, 
              pos=wx.Point(col2,10),
              size=wx.Size(box_dimensions),style = wx.ALIGN_LEFT)
        ## X
        self.text_x = wx.StaticText(id=wxID_text_x,
              label=u'x-pos.', name='text_x', parent=self.panel1, 
              pos=wx.Point(col2, 30),
              size=wx.Size(medium_box_dimensions),style = wx.ALIGN_LEFT)
        self.input_x = wx.TextCtrl(id=wxID_input_x,
              name=u'input_x', parent=self.panel1, 
              style=0, value=u'0',
              pos=wx.Point(col2 + 55, 30),size=wx.Size(medium_box_dimensions))
        
        ## Y      
        self.text_y = wx.StaticText(id=wxID_text_y,
              label=u'y-pos.', name='text_y', parent=self.panel1,
              pos=wx.Point(col2, 55), size=wx.Size(medium_box_dimensions), style=wx.ALIGN_LEFT)
        self.input_y = wx.TextCtrl(id=wxID_input_y,
              name=u'input_y', parent=self.panel1, pos=wx.Point(col2 + 55,55),
              size=wx.Size(medium_box_dimensions), style=0, value=u'0')
        
        ## Width
        self.text_w = wx.StaticText(id=wxID_text_w,
              label=u'Width:', name='text_w', parent=self.panel1,
              pos=wx.Point(col2, 80), size=wx.Size(medium_box_dimensions), style=wx.ALIGN_LEFT)
        self.input_w = wx.TextCtrl(id=wxID_input_w,
              name=u'input_w', parent=self.panel1, pos=wx.Point(col2 + 55, 80),
              size=wx.Size(medium_box_dimensions), style=0, value=u'0')
        
        ## Height
        self.text_h = wx.StaticText(id=wxID_text_h,
              label=u'Height:', name='text_h', parent=self.panel1,
              pos=wx.Point(col2,105), size=wx.Size(medium_box_dimensions), style=wx.ALIGN_LEFT)
        self.input_h = wx.TextCtrl(id=wxID_input_h,
              name=u'input_h', parent=self.panel1, pos=wx.Point(col2 + 55, 105),
               size=wx.Size(medium_box_dimensions), style=0, value=u'0')
        
        ## ROI rectangle stays same dimensions but can be redrawn. Not critical to keep
        self.checkBox_fixed_ROI = wx.CheckBox(id=wxID_check_box_ROI,
              label=u'Fixed ROI Size?', name=u'checkBox_fixed_ROI',
              parent=self.panel1, pos=wx.Point(col2, 130), size=wx.Size(250,22),
              style=0)
        self.checkBox_fixed_ROI.SetValue(False)
        ######
        
       ## Detection parameters

        self.text_step_3 = wx.StaticText(id=wxID_text_step_3,
              label=u'Step 3: Specify spot parameters', name='text_step_3', parent=self.panel1, 
              pos=wx.Point(col3,10),
              size=wx.Size(100,22),style = wx.ALIGN_LEFT)
                      
        ## Expected spot diameter
        self.text_diameter = wx.StaticText(id=wxID_text_diameter,
              label=u'Diameter:', name='text_diameter', parent=self.panel1,
              pos=wx.Point(col3 ,30), size=wx.Size(medium_box_dimensions), style=0)
        self.input_diameter = wx.TextCtrl(id=wxID_input_diameter,
              name=u'input_diameter', parent=self.panel1, pos=wx.Point(col3 + 90,30), 
              size=wx.Size(medium_box_dimensions), style=0, value=u'7')
        
        ## Maximum spot diameter
        self.text_maxsize = wx.StaticText(id=wxID_text_maxsize,
              label=u'MaxDiameter:', name='text_maxsize', parent=self.panel1,
              pos=wx.Point(col3,55), size=wx.Size(medium_box_dimensions), style=0)
        self.input_maxsize = wx.TextCtrl(id=wxID_input_maxsize,
              name=u'input_maxsize', parent=self.panel1, pos=wx.Point(col3 + 90,55), 
              size=wx.Size(medium_box_dimensions), style=0, value=u'11')

        ## Minimum spot 'mass'
        self.text_threshold = wx.StaticText(id=wxID_text_minmass,
              label=u'MinMass:', name='text_threshold', parent=self.panel1,
              pos=wx.Point(col3,80), size=wx.Size(medium_box_dimensions), style=0)
        self.input_minmass = wx.TextCtrl(id=wxID_input_minmass,
              name=u'input_minmass', parent=self.panel1, pos=wx.Point(col3 + 90,80), 
              size=wx.Size(medium_box_dimensions), style=0, value=u'100')

        ## Spot threshold
        self.text_threshold = wx.StaticText(id=wxID_text_threshold,
              label=u'Threshold:', name='text_threshold', parent=self.panel1,
              pos=wx.Point(col3, 105), size=wx.Size(medium_box_dimensions), style=0)
        self.input_threshold = wx.TextCtrl(id=wxID_input_threshold,
              name=u'input_threshold', parent=self.panel1, pos=wx.Point(col3 + 90,105),
              size=wx.Size(medium_box_dimensions), style=0, value=u'"auto"')
        
        ## Check frames
        self.text_step_4 = wx.StaticText(id=wxID_text_step_4,
              label=u'Step 4: Additional parameters', name='text_step_4', parent=self.panel1, 
              pos=wx.Point(col4,10),
              size=wx.Size(100,22),style = wx.ALIGN_LEFT)

        self.text_check_frames = wx.StaticText(id=wxID_text_check_frames,
              label=u'Check Frames:', name='text_check_frames', parent=self.panel1,
              pos=wx.Point(col4, 55), size=wx.Size(115, 17), style=0)

        self.input_frame_0 = wx.TextCtrl(id=input_frame_0,
              name=u'input_frame_0', parent=self.panel1, pos=wx.Point(col4 + 130, 55),
               size=wx.Size(small_box_dimensions), style=0, value=u'')

        self.input_frame_n = wx.TextCtrl(id=wxID_input_frame_n,
              name=u'input_frame_n', parent=self.panel1, pos=wx.Point(col4 + 170, 55), 
              size=wx.Size(small_box_dimensions), style=0, value=u'')
        
        ## Background frames
        self.text_background_frames = wx.StaticText(id=wxID_text_background_frames,
              label=u'Background Frames:', name='text_background_frames',
              parent=self.panel1, pos=wx.Point(col4, 30), size=wx.Size(medium_box_dimensions),
              style=0)
        self.input_blank_0 = wx.TextCtrl(id=wxID_input_blank_0,
              name=u'input_blank_0', parent=self.panel1, pos=wx.Point(col4+ 130, 30),
            size=wx.Size(small_box_dimensions), style=0, value=u'')

        self.input_blank_n = wx.TextCtrl(id=wxID_input_blank_n,
              name=u'input_blank_n', parent=self.panel1, pos=wx.Point(col4 + 170,30),
              size=wx.Size(small_box_dimensions), style=0, value=u'')
        
        ## Vials
        self.text_vials = wx.StaticText(id=wxID_text_vials,
              label=u'Number of vials:', name='text_vials',
              parent=self.panel1, pos=wx.Point(col4, 80), size=wx.Size(133, 22),
              style=0)
        self.input_vials = wx.TextCtrl(id=wxID_input_vials,
              name=u'input_vials', parent=self.panel1, pos=wx.Point(col4 + 130, 80), 
              size=wx.Size(small_box_dimensions), style=0, value=u'1')
        
        ## Window size
        self.text_window = wx.StaticText(id=wxID_text_window,
              label=u'Window size:', name='text_window',
              parent=self.panel1, pos=wx.Point(col4, 105), size=wx.Size(133, 22),
              style=0)
        self.input_window = wx.TextCtrl(id=wxID_input_window,
              name=u'input_window', parent=self.panel1, pos=wx.Point(col4 + 130, 105), 
              size=wx.Size(small_box_dimensions), style=0, value='1')    
          

        self.text_step_5 = wx.StaticText(id=wxID_text_step_5,
              label=u'Step 5: Naming parameters', name='text_step_5', parent=self.panel1, 
              pos=wx.Point(col5,10),
              size=wx.Size(100,22),style = wx.ALIGN_LEFT)

        ## Naming convention
        self.text_naming_convention = wx.StaticText(id=wxID_text_naming_convention,
              label=u"Naming pattern:", name='text_naming_convention',
              parent=self.panel1, pos=wx.Point(col5, 30), size=wx.Size(medium_box_dimensions),
              style=0)
        self.input_naming_convention = wx.TextCtrl(id=wxID_input_naming_convention,
              name=u'input_naming_convention', parent=self.panel1, pos=wx.Point(col5 + 20, 50), 
              size=wx.Size(large_box_dimensions), style=0, value='') 
        
        ## Variables
        self.text_vvars = wx.StaticText(id=wxID_text_vvars,
              label=u'Video vars:', name='text_vvars',
              parent=self.panel1, pos=wx.Point(col5, 80), size=wx.Size(133, 22),
              style=0)
        self.input_vvars = wx.TextCtrl(id=wxID_input_vvars,
              name=u'input_vvars', parent=self.panel1, pos=wx.Point(col5 + 130, 80), 
              size=wx.Size(small_box_dimensions), style=0, value=u'2')  
        
        self.text_path_project = wx.StaticText(id=wxID_text_path_project,
              label=u"Project path:", name='text_path_project',
              parent=self.panel1, pos=wx.Point(col5, 105), size=wx.Size(medium_box_dimensions),
              style=0)
        self.input_path_project = wx.TextCtrl(id=wxID_input_path_project,
              name=u'input_path_project', parent=self.panel1, pos=wx.Point(col5, 125), 
              size=wx.Size(large_box_dimensions), style=0, value='') 


        ## Bottom panels
        self.text_video_path = wx.StaticText(id=wxID_video_path,
              label='Video Path', name='text_video_path', parent=self.panel1,
              pos=wx.Point(10, 205), size=wx.Size(930, 22), style=0)
        self.text_video_path.SetBackgroundColour(wx.Colour(241, 241, 241))

        self.button_test_parameters = wx.Button(id=wxID_test_parameters,
              label=u'Test parameters', name=u'button_test_parameters',
              parent=self.panel1, pos=wx.Point(col1,180), size=wx.Size(140, 22),
              style=wx.ALIGN_CENTER)
        self.button_test_parameters.Bind(wx.EVT_BUTTON, self.OnButton_testParButton,
              id=wxID_test_parameters)

        self.button_reload_video = wx.Button(id=wxID_reload_video,
              label=u'Reload video', name=u'button_reload_video', parent=self.panel1,
              pos=wx.Point(col1 + 160*1, 180), size=wx.Size(140, 22), style=wx.ALIGN_CENTER)
        self.button_reload_video.Bind(wx.EVT_BUTTON, self.OnButton_LoadVideo,
              id=wxID_reload_video)
              
        self.button_store_parameters = wx.Button(id=wxID_store_parameters,
              label=u'Save configuration', name=u'button_store_parameters',
              parent=self.panel1, pos=wx.Point(col1 + 160*2, 180), size=wx.Size(140, 22),
              style=wx.ALIGN_CENTER)
        self.button_store_parameters.Bind(wx.EVT_BUTTON, self.OnButton_strParButton,
              id=wxID_store_parameters)

        self.button_run_analysis = wx.Button(id=wxID_run_analysis,
              label=u'Run analysis', name=u'button_run_analysis', parent=self.panel1,
              pos=wx.Point(col1 + 160*3, 180), size=wx.Size(140, 22), style=wx.ALIGN_CENTER)
        self.button_run_analysis.Bind(wx.EVT_BUTTON, self.OnButton_runAnaButton,
              id=wxID_run_analysis)

#         self.button_display_parameters = wx.Button(id=wxID_display_parameters,
#               label=u'Display Parameters', name=u'button_display_parameters', parent=self.panel1,
#               pos=wx.Point(col1 + 160*4, 180), size=wx.Size(140, 22), style=wx.ALIGN_CENTER)
#         self.button_display_parameters.Bind(wx.EVT_BUTTON, self.OnButton_display_parameters,
#               id=wxID_display_parameters)
              
        self.status_bar = wx.StatusBar(id=wxID_status_bar,
              name='status_bar', parent=self, style=0)
        self._init_sizers()


[wxID_text_step_1a,wxID_text_step_1b,wxID_text_step_2,wxID_text_step_3,wxID_text_step_4,wxID_text_step_5,
wxID_panel_1, wxID_text_title, 
wxID_browse_video, wxID_reload_video,
wxID_run_analysis, wxID_store_parameters,
wxID_test_parameters, wxID_display_parameters,
wxID_video_path, 
wxID_frame_rate,
wxID_status_bar, 
wxID_text_minmass,wxID_input_minmass, 
wxID_text_threshold, wxID_input_threshold,
wxID_text_maxsize, wxID_input_maxsize,
wxID_text_diameter, wxID_input_diameter,
wxID_text_x, wxID_input_x,
wxID_text_y, wxID_input_y,
wxID_text_h, wxID_input_h,
wxID_text_w, wxID_input_w,
wxID_text_check_frames, input_frame_0, wxID_input_frame_n,
wxID_text_background_frames,wxID_input_blank_0, wxID_input_blank_n,
wxID_text_vials, wxID_input_vials,
wxID_text_window, wxID_input_window,
wxID_text_pixel_to_cm,wxID_input_pixel_to_cm,
wxID_text_naming_convention,wxID_input_naming_convention,
wxID_text_vvars,wxID_input_vvars,
wxID_text_path_project,wxID_input_path_project,
wxID_input_convert_to_cm,wxID_check_box_ROI,
wxID_vert_line_1] = [wx.NewId() for _init_controls in range(54)]


## Basic GUI sizes and spacers
col1,col2,col3,col4,col5 = 10,180,320,525,750

small_box_dimensions = 35,22
medium_box_dimensions = 50,22
box_dimensions = 100,22
large_box_dimensions = 145,22

def create(parent, file_path):
    print('-------->  create')
    if file_path==None:
        file_path='Select a video to begin'
    return DectFrame(parent, file_path)
